#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os
import re
import json
import codecs
import requests
import pdb
import webbrowser
from collections import defaultdict
from chgksuite.common import (
    get_lastdir, set_lastdir, on_close, log_wrap, bring_to_front,
    get_chgksuite_dir
)

API = 'https://trello.com/1'
re_bi = re.compile(r'trello\.com/b/(.+?)(/|$)')



def upload_file(filepath, trello):
    req = requests.get("{}/boards/{}/lists".format(API, trello['board_id']),
                       params={'token': trello['params']['token'],
                               'key': trello['params']['key']})
    if req.status_code != 200:
        print('Error: {}'.format(req.text))
        sys.exit(1)

    lists = json.loads(req.content.decode('utf8'))
    lid = lists[0]['id']
    content = ''
    with codecs.open(filepath, 'r', 'utf8') as f:
        content = f.read()
    cards = re.split(r'(\r?\n){2,}', content)
    cards = [x for x in cards if x != '' and x != '\n' and x != '\r\n']
    for card in cards:
        caption = 'вопрос'
        if re.search('\n! (.+?)\r?\n', card):
            caption = re.search('\n! (.+?)\.?\r?\n', card).group(1)
            if trello['author'] and re.search('\n@ (.+?)\.?\r?\n', card):
                caption += ' {}'.format(
                    re.search('\n@ (.+?)\r?\n', card).group(1))

        req = requests.post(
            "{}/lists/{}/cards".format(API, lid),
            {
                'key': trello['params']['key'],
                'token': trello['params']['token'],
                'desc': card,
                'name': caption
            })
        if req.status_code == 200:
            print('Successfully sent {}'.format(log_wrap(caption)))
        else:
            print('Error {}: {}'.format(req.status_code, req.content))


def gui_trello_upload(args, sourcedir):
    ld = get_lastdir()

    if not args.board_id:
        board_id = get_board_id()
    else:
        board_id = args.board_id

    trelloconfig = args.trelloconfig
    trelloconfig['board_id'] = board_id

    trelloconfig['author'] = args.author

    if isinstance(args.filename, (list, tuple)):
        if len(args.filename) == 1 and os.path.isdir(args.filename[0]):
            for filename in os.listdir(args.filename[0]):
                if filename.endswith('.4s'):
                    filepath = os.path.join(args.filename[0], filename)
                    upload_file(filepath, trelloconfig)
            set_lastdir(args.filename[0])
        else:
            for filename in args.filename:
                upload_file(filename, trelloconfig)
                set_lastdir(filename)
    elif isinstance(args.filename, str):
        if os.path.isdir(args.filename):
            for filename in os.listdir(args.filename):
                if filename.endswith('.4s'):
                    filepath = os.path.join(args.filename, filename)
                    upload_file(filepath, trelloconfig)
                    set_lastdir(filepath)
        elif os.path.isfile(args.filename):
            upload_file(args.filename, trelloconfig)
            set_lastdir(args.filename)


def onlyanswers_line_check(line):
    line = (line or '')
    return line.startswith(
        ('Ответ', 'Зачёт', 'Зачет', '1', '2', '3', '4', '5', '6', '8')
    )


def noanswers_line_check(line):
    line = (line or '')
    return not line.startswith(
        (
            'Ответ', 'Коммента', 'Источник', 'Автор', 'Зачёт', 'Зачет',
            'Незачёт', 'Незачет'
        )
    )


def process_desc(s, onlyanswers=False, noanswers=False):
    s = s.strip()
    s = s.replace(r'\`', '`')
    s = s.replace(r'\*', '*')
    if onlyanswers:
        lines = s.split('\n')
        lines = [
            x for x in lines if onlyanswers_line_check(x)
        ]
        s = '\n'.join(lines)
    elif noanswers:
        lines = s.split('\n')
        lines = [
            x for x in lines if noanswers_line_check(x)
        ]
        s = '\n'.join(lines)
    return s


def getlabels(s):
    return {x['name'] for x in s['labels']}


def delete_paragraph(paragraph):
    p = paragraph._element
    p.getparent().remove(p)
    p._p = p._element = None


def add_themes_list(group):
    themes = group['themes']
    p = group['paragraph']
    if len(themes):
        p.add_run('Темы: ' + ', '.join(themes) + '.').bold = True
    else:
        delete_paragraph(p)


def gui_trello_download(args, sourcedir):
    ld = get_lastdir()

    template_path = args.docx_template

    board_id_path = os.path.join(args.folder, '.board_id')
    if os.path.isfile(board_id_path):
        with codecs.open(board_id_path, 'r', 'utf8') as f:
            board_id = f.read().rstrip()
    else:
        board_id = get_board_id(path=args.folder)

    params = args.trelloconfig['params']
    ld = args.folder
    set_lastdir(ld)
    os.chdir(args.folder)

    if args.si or args.qb:
        from docx import Document

    req = requests.get("{}/boards/{}".format(API, board_id),
                       params=params)
    if req.status_code != 200:
        print('Error: {}'.format(req.text))
        if args.debug:
            pdb.set_trace()
        sys.exit(1)

    _lists = defaultdict(lambda: [])
    _list_counters = defaultdict(lambda: 0)

    json_ = json.loads(req.content.decode('utf8'))
    _names = defaultdict(lambda: None)
    open_lists = list(filter(lambda l: not l['closed'], json_['lists']))
    for list_ in open_lists:
        _names[list_['id']] = list_['name'].replace('/', '_')
        _list_counters[list_['id']] = 0

    if args.si:
        _docs = defaultdict(lambda: Document(template_path))
        _groups = defaultdict(lambda: None)
    if args.qb:
        qb_doc = Document(template_path)

    for card in json_['cards']:
        card["desc"] = card["desc"].replace("\n\n", "\n")
        list_id = card['idList']
        list_name = _names[list_id]
        if card.get('closed') or list_name is None:
            continue

        _list_counters[list_id] += 1

        if not args.si:
            card_title = ''
        elif card['name'].startswith('#'):
            card_title = card['name']
            _list_counters[list_id] = 0
        else:
            card_title = 'Тема {}. {}'.format(
                _list_counters[list_id], card['name']
            )
            clear_card_title = card['name']

        id_ = ('singlefile' if args.singlefile else list_name)

        if args.si:
            doc_ = _docs[id_]
            group_ = _groups[id_]
            if group_ is None:  # new doc
                group_ = _groups[id_] = {
                    'paragraph': doc_.add_paragraph(),
                    'themes': []
                }
            if card_title:  # new title
                if card_title.startswith('#'):
                    title_re = r'(#+)\s*(.*)'
                    m = re.search(title_re, card_title)
                    doc_.add_heading(m[2], level=len(m[1]))
                    add_themes_list(group_)
                    group_['paragraph'] = doc_.add_paragraph()
                    group_['themes'] = []
                    doc_.add_paragraph()
                else:
                    group_['themes'].append(clear_card_title)
                    p = doc_.add_paragraph()
                    p.add_run(card_title).bold = True
                    doc_.add_paragraph()
            if card['desc']:
                doc_.add_paragraph(process_desc(
                    card['desc'],
                    onlyanswers=args.onlyanswers,
                    noanswers=args.noanswers,
                ))

        _lists[id_].append(card_title + ('' if card_title.startswith('#')
                                         else '\n\n') + process_desc(card['desc']))

        if args.labels:
            for label in getlabels(card):
                _lists[label].append(
                    (card['name'] if args.si else '') +
                    process_desc(card['desc']))
    if args.si:
        for list_name in _groups:
            # add remaining themes when we know we're done with the doc
            add_themes_list(_groups[list_name])
        for list_name in _docs:
            _docs[list_name].save('{}.docx'.format(list_name))

    if args.qb:
        first, second = _lists[args.qb[0]], _lists[args.qb[1]]
        for i, pair in enumerate(zip(first, second)):
            p = qb_doc.add_paragraph()
            p.add_run('Тоссап {}.'.format(i + 1)).bold = True
            p = qb_doc.add_paragraph()
            p = qb_doc.add_paragraph()
            p.add_run(pair[0])
            p = qb_doc.add_paragraph()
            p = qb_doc.add_paragraph()
            p.add_run('Бонус {}.'.format(i + 1)).bold = True
            p = qb_doc.add_paragraph()
            p = qb_doc.add_paragraph()
            p.add_run(pair[1])
            p = qb_doc.add_paragraph()
            p = qb_doc.add_paragraph()
        qb_doc.save('quizbowl.docx')

    for _list in _lists:
        filename = '{}.4s'.format(_list)
        print('outputting {}'.format(filename))
        with codecs.open(filename, 'w', 'utf8') as f:
            for item in _lists[_list]:
                f.write('\n' + item + '\n')


def get_board_id(path=None):
    print('To communicate with your trello board we need its board_id.')
    print('Your board link looks like this:')
    print()
    print('https://trello.com/b/Bi0z2H49/title-of-your-board')
    print('                     board_id')
    print()
    board_id = input('Please paste your board_id '
                     '(or the board link, '
                     'we\'ll parse it): ').rstrip()
    if 'trello.com' in board_id:
        board_id = re_bi.search(board_id).group(1)
    if path:
        with codecs.open(os.path.join(path, '.board_id'), 'w', 'utf8') as f:
            f.write(board_id)
    return board_id


def get_token(tokenpath):
    webbrowser.open('https://trello.com/1/connect'
                    '?key=1d4fe71dd193855686196e7768aa4b05'
                    '&name=Chgk&scope=read,write&response_type=token')
    token = input('Please paste the obtained token: ').rstrip()
    with codecs.open(tokenpath, 'w', 'utf8') as f:
        f.write(token)
    return token


def gui_trello(args, sourcedir=None):
    csdir = get_chgksuite_dir()
    resourcedir = os.path.join(sourcedir, "resources")
    tokenpath = os.path.join(csdir, '.trello_token')
    if not os.path.isfile(tokenpath):
        token = get_token(tokenpath)
    else:
        with codecs.open(tokenpath, 'r', 'utf8') as f:
            token = f.read().rstrip()

    with open(os.path.join(resourcedir, 'trello.json')) as f:
        args.trelloconfig = json.load(f)
    args.trelloconfig['params']['token'] = token

    if args.trellosubcommand == 'download':
        gui_trello_download(args, sourcedir)
    elif args.trellosubcommand == 'upload':
        gui_trello_upload(args, sourcedir)
    elif args.trellosubcommand == 'token':
        get_token(tokenpath)


def main():
    print('This program was not designed to run standalone.')
    input("Press Enter to continue...")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import sys
import codecs
import argparse
import time
import json
import traceback


QUESTION_LABELS = ['handout', 'question', 'answer',
                   'zachet', 'nezachet', 'comment',
                   'source', 'author', 'number',
                   'setcounter']
SEP = os.linesep
ENC = sys.stdout.encoding or 'utf8'

lastdir = os.path.join(os.path.dirname(os.path.abspath('__file__')),
                       'lastdir')


def get_chgksuite_dir():
    chgksuite_dir = os.path.join(os.path.expanduser("~"), ".chgksuite")
    if not os.path.isdir(chgksuite_dir):
        os.mkdir(chgksuite_dir)
    return chgksuite_dir


def get_source_dirs():
    if getattr(sys, "frozen", False):
        sourcedir = os.path.dirname(sys.executable)
        resourcedir = os.path.join(sourcedir, "resources")
    else:
        sourcedir = os.path.dirname(
            os.path.abspath(os.path.realpath(__file__))
        )
        resourcedir = os.path.join(sourcedir, "resources")
    return sourcedir, resourcedir


def set_lastdir(path):
    chgksuite_dir = get_chgksuite_dir()
    lastdir = os.path.join(chgksuite_dir, "lastdir")
    with codecs.open(lastdir, 'w', 'utf8') as f:
        f.write(path)


def bring_to_front(root):
    root.lift()
    root.attributes('-topmost', True)
    root.after_idle(root.attributes, '-topmost', False)


def get_lastdir():
    chgksuite_dir = get_chgksuite_dir()
    lastdir = os.path.join(chgksuite_dir, "lastdir")
    if os.path.isfile(lastdir):
        with codecs.open(lastdir, 'r', 'utf8') as f:
            return f.read().rstrip()
    return '.'


def retry_wrapper_factory(logger):
    def retry_wrapper(func, args=None, kwargs=None, retries=3):
        cntr = 0
        ret = None
        if not args:
            args = []
        if not kwargs:
            kwargs = {}
        while not ret and cntr < retries:
            try:
                ret = func(*args, **kwargs)
            except:
                logger.error(traceback.format_exc())
                time.sleep(5)
                cntr += 1
        return ret
    return retry_wrapper


def ensure_utf8(s):
    if isinstance(s, bytes):
        return s.decode("utf8", errors="replace")
    return s


class DummyLogger(object):

    def info(self, s):
        pass

    def debug(self, s):
        pass

    def error(self, s):
        pass

    def warning(self, s):
        pass


class DefaultNamespace(argparse.Namespace):

    def __init__(self, *args, **kwargs):
        for ns in args:
            if isinstance(ns, argparse.Namespace):
                for name in vars(ns):
                    setattr(self, name, vars(ns)[name])
        else:
            for name in kwargs:
                setattr(self, name, kwargs[name])

    def __getattribute__(self, name):
        try:
            return argparse.Namespace.__getattribute__(self, name)
        except AttributeError:
            return


def on_close(root):
    root.quit()
    root.destroy()


def log_wrap(s, pretty_print=True):
    try_to_unescape = True
    if pretty_print and isinstance(s, (dict, list)):
        s = json.dumps(s, indent=2, ensure_ascii=False, sort_keys=True)
        try_to_unescape = False
    s = format(s)
    if sys.version_info.major == 2 and try_to_unescape:
        try:
            s = s.decode('unicode_escape')
        except UnicodeEncodeError:
            pass
    return s.encode(ENC, errors='replace').decode(ENC)


def toggle_factory(intvar, strvar, root):
    def toggle():
        if intvar.get() == 0:
            intvar.set(1)
        else:
            intvar.set(0)
        root.ret[strvar] = bool(intvar.get())
    return toggle


def button_factory(strvar, value, root):
    def button():
        root.ret[strvar] = value
        root.quit()
        root.destroy()
    return button


def check_question(question, logger=None):
    warnings = []
    for el in {'question', 'answer', 'source', 'author'}:
        if el not in question:
            warnings.append(el)
    if len(warnings) > 0:
        logger.warning('WARNING: question {} lacks the following fields: {}{}'
                       .format(log_wrap(question), ', '.join(warnings), SEP))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import argparse
import os
import sys

try:
    import Tkinter as tk
    import Tkinter.filedialog as filedialog
    import Tkinter.ttk as ttk
except ImportError:
    import tkinter as tk
    import tkinter.filedialog as filedialog
    import tkinter.ttk as ttk

from chgksuite.parser import gui_parse
from chgksuite.composer import gui_compose
from chgksuite.trello import gui_trello
from chgksuite.version import __version__
from chgksuite.common import (
    on_close,
    button_factory,
    toggle_factory,
    DefaultNamespace,
    bring_to_front,
    get_lastdir,
    ensure_utf8,
    get_source_dirs,
)

from collections import defaultdict
import json

try:
    basestring
except NameError:
    basestring = (str, bytes)

debug = False


class VarWrapper(object):
    def __init__(self, name, var):
        self.name = name
        self.var = var


class OpenFileDialog(object):
    def __init__(self, label, var, folder=False, lastdir=None, filetypes=None):
        self.label = label
        self.var = var
        self.folder = folder
        self.lastdir = lastdir
        self.filetypes = filetypes

    def __call__(self):
        function = (
            filedialog.askdirectory if self.folder else filedialog.askopenfilename
        )
        kwargs = {}
        if self.lastdir:
            kwargs["initialdir"] = self.lastdir
        if self.filetypes:
            kwargs["filetypes"] = self.filetypes
        output = function(**kwargs)
        if isinstance(output, bytes):
            output = output.decode("utf8")
        self.var.set(output or "")
        self.label.config(text=(output or "").split(ensure_utf8(os.sep))[-1])


class ParserWrapper(object):
    def __init__(self, parser, parent=None, lastdir=None):
        self.parent = parent
        if self.parent and not lastdir:
            self.lastdir = self.parent.lastdir
        else:
            self.lastdir = lastdir
        if self.parent:
            self.parent.children.append(self)
            self.frame = tk.Frame(self.parent.frame)
            self.frame.pack()
            self.frame.pack_forget()
            self.advanced_frame = tk.Frame(self.parent.advanced_frame)
            self.advanced_frame.pack()
            self.advanced_frame.pack_forget()
        else:
            self.init_tk()
        self.parser = parser
        self.subparsers_var = None
        self.cmdline_call = None
        self.children = []
        self.vars = []

    def _list_vars(self):
        result = []
        for var in self.vars:
            result.append((var.name, var.var.get()))
        if self.subparsers_var:
            chosen_parser_name = self.subparsers_var.get()
            chosen_parser = [
                x
                for x in self.subparsers.parsers
                if x.parser.prog.split()[-1] == chosen_parser_name
            ][0]
            result.append(("", chosen_parser_name))
            result.extend(chosen_parser._list_vars())
        return result

    def build_command_line_call(self):
        result = []
        result_to_print = []
        for tup in self._list_vars():
            to_append = None
            if tup[0].startswith("--"):
                if tup[1] == "true":
                    to_append = tup[0]
                elif not tup[1] or tup[1] == "false":
                    continue
                else:
                    to_append = [tup[0], tup[1]]
            else:
                to_append = tup[1]
            if isinstance(to_append, list):
                result.extend(to_append)
                if "password" in tup[0]:
                    result_to_print.append(tup[0])
                    result_to_print.append("********")
                else:
                    result_to_print.extend(to_append)
            else:
                result.append(to_append)
                result_to_print.append(to_append)
        print("Command line call: {}".format(" ".join(result_to_print)))
        return result

    def ok_button_press(self):
        self.cmdline_call = self.build_command_line_call()
        self.tk.quit()
        self.tk.destroy()

    def toggle_advanced_frame(self):
        value = self.advanced_checkbox_var.get()
        if value == "true":
            self.advanced_frame.pack()
        else:
            self.advanced_frame.pack_forget()

    def init_tk(self):
        self.tk = tk.Tk()
        self.tk.title("chgksuite v{}".format(__version__))
        self.tk.minsize(600, 300)
        self.tk.eval("tk::PlaceWindow . center")
        self.mainframe = tk.Frame(self.tk)
        self.mainframe.pack(side="top")
        self.frame = tk.Frame(self.mainframe)
        self.frame.pack(side="top")
        self.button_frame = tk.Frame(self.mainframe)
        self.button_frame.pack(side="top")
        self.ok_button = tk.Button(
            self.button_frame,
            text="Запустить",
            command=self.ok_button_press,
            width=15,
            height=2,
        )
        self.ok_button.pack(side="top")
        self.advanced_checkbox_var = tk.StringVar()
        self.toggle_advanced_checkbox = tk.Checkbutton(
            self.button_frame,
            text="Показать дополнительные настройки",
            onvalue="true",
            offvalue="false",
            variable=self.advanced_checkbox_var,
            command=self.toggle_advanced_frame,
        )
        self.toggle_advanced_checkbox.pack(side="top")
        self.advanced_frame = tk.Frame(self.mainframe)
        self.advanced_frame.pack(side="top")
        self.advanced_frame.pack_forget()

    def add_argument(self, *args, **kwargs):

        if kwargs.pop("advanced", False):
            frame = self.advanced_frame
        else:
            frame = self.frame
        if kwargs.pop("hide", False):
            self.parser.add_argument(*args, **kwargs)
            return
        caption = kwargs.pop("caption", None) or args[0]
        argtype = kwargs.pop("argtype", None)
        filetypes = kwargs.pop("filetypes", None)
        if not argtype:
            if kwargs.get("action") == "store_true":
                argtype = "checkbutton"
            elif args[0] in {"filename", "folder"}:
                argtype = args[0]
            else:
                argtype = "entry"
        if argtype == "checkbutton":
            var = tk.StringVar()
            var.set("false")
            innerframe = tk.Frame(frame)
            innerframe.pack(side="top")
            checkbutton = tk.Checkbutton(
                innerframe, text=caption, variable=var, onvalue="true", offvalue="false"
            )
            checkbutton.pack(side="left")
            self.vars.append(VarWrapper(name=args[0], var=var))
        elif argtype in {"filename", "folder"}:
            text = "(имя файла)" if argtype == "filename" else "(имя папки)"
            button_text = "Открыть файл" if argtype == "filename" else "Открыть папку"
            var = tk.StringVar()
            innerframe = tk.Frame(frame)
            innerframe.pack(side="top")
            label = tk.Label(innerframe, text=caption)
            label.pack(side="left")
            label = tk.Label(innerframe, text=text)
            ofd_kwargs = dict(folder=argtype == "folder", lastdir=self.lastdir)
            if filetypes:
                ofd_kwargs["filetypes"] = filetypes
            button = tk.Button(
                innerframe,
                text=button_text,
                command=OpenFileDialog(label, var, **ofd_kwargs),
            )
            button.pack(side="left")
            label.pack(side="left")
            self.vars.append(VarWrapper(name=args[0], var=var))
        elif argtype == "entry":
            var = tk.StringVar()
            var.set(kwargs.get("default") or "")
            innerframe = tk.Frame(frame)
            innerframe.pack(side="top")
            tk.Label(innerframe, text=caption).pack(side="left")
            entry_show = "*" if "password" in args[0] else ""
            entry = tk.Entry(innerframe, textvariable=var, show=entry_show)
            entry.pack(side="left")
            self.vars.append(VarWrapper(name=args[0], var=var))
        self.parser.add_argument(*args, **kwargs)

    def add_subparsers(self, *args, **kwargs):
        subparsers = self.parser.add_subparsers(*args, **kwargs)
        self.subparsers_var = tk.StringVar()
        self.subparsers = SubparsersWrapper(subparsers, parent=self)
        return self.subparsers

    def show_frame(self):
        for child in self.parent.children:
            child.frame.pack_forget()
            child.advanced_frame.pack_forget()
        self.frame.pack(side="top")
        self.advanced_frame.pack(side="top")

    def parse_args(self, *args, **kwargs):
        argv = sys.argv[1:]
        if not argv:
            self.tk.mainloop()
            if self.cmdline_call:
                return DefaultNamespace(self.parser.parse_args(self.cmdline_call))
            else:
                sys.exit(0)
        return DefaultNamespace(self.parser.parse_args(*args, **kwargs))


class SubparsersWrapper(object):
    def __init__(self, subparsers, parent):
        self.subparsers = subparsers
        self.parent = parent
        self.frame = tk.Frame(self.parent.frame)
        self.frame.pack(side="top")
        self.parsers = []

    def add_parser(self, *args, **kwargs):
        caption = kwargs.pop("caption", None) or args[0]
        parser = self.subparsers.add_parser(*args, **kwargs)
        pw = ParserWrapper(parser=parser, parent=self.parent)
        self.parsers.append(pw)
        radio = tk.Radiobutton(
            self.frame,
            text=caption,
            variable=self.parent.subparsers_var,
            value=args[0],
            command=pw.show_frame,
        )
        radio.pack(side="left")
        return pw


def app():
    sourcedir, resourcedir = get_source_dirs()

    if isinstance(sourcedir, bytes):
        sourcedir = sourcedir.decode("utf8")
    ld = get_lastdir()
    parser = ParserWrapper(
        argparse.ArgumentParser(prog="chgksuite".format(__version__)), lastdir=ld
    )
    parser.add_argument(
        "--debug",
        "-d",
        action="store_true",
        help="Print and save some debug info.",
        caption="Отладочная информация",
        advanced=True,
    )
    parser.add_argument(
        "--config",
        "-c",
        help="a config file to store default args values.",
        caption="Файл конфигурации",
        advanced=True,
        argtype="filename",
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version="%(prog)s " + __version__,
        hide=True,
    )
    subparsers = parser.add_subparsers(dest="action")

    cmdparse = subparsers.add_parser("parse")
    cmdparse.add_argument(
        "filename",
        help="file to parse.",
        nargs="?",
        caption="Имя файла",
        filetypes=[("chgksuite parsable files", ("*.docx", "*.txt"))],
    )
    cmdparse.add_argument(
        "--defaultauthor",
        action="store_true",
        help="pick default author from filename " "where author is missing.",
        advanced=True,
        caption="Дописать отсутствующего автора из имени файла",
    )
    cmdparse.add_argument(
        "--encoding",
        default=None,
        help="Encoding of text file " "(use if auto-detect fails).",
        advanced=True,
        caption="Кодировка",
    )
    cmdparse.add_argument(
        "--regexes",
        default=None,
        help="A file containing regexes " "(the default is regexes.json).",
        advanced=True,
        caption="Файл с регулярными выражениями",
        argtype="filename",
    )
    cmdparse.add_argument(
        "--parsedir",
        action="store_true",
        help="parse directory instead of file.",
        advanced=True,
        hide=True,
    )
    cmdparse.add_argument(
        "--links",
        default="unwrap",
        choices=["unwrap", "old"],
        help="hyperlinks handling strategy. "
        "Unwrap just leaves links as presented in the text, unchanged. "
        "Old is behaviour from versions up to v0.5.3: "
        "replace link with its href value.",
        advanced=True,
        caption="Стратегия обработки ссылок",
    )
    cmdparse.add_argument(
        "--numbers_handling",
        default="default",
        choices=["default", "all", "none"],
        help="question numbers handling strategy. "
        "Default preserves zero questions and numbering "
        "if the first question has number > 1, omits number otherwise. "
        "All preserves all numbers, none omits all numbers "
        "(was default behaviour pre-0.8.0.)",
        advanced=True,
        caption="Стратегия обработки номеров вопросов",
    )
    cmdparse.add_argument(
        "--fix_spans",
        action="store_true",
        help="try to unwrap all <span> tags. " "Can help fix weird Word formatting.",
        advanced=True,
        caption="Fix <span> tags",
    )
    cmdparse.add_argument(
        "--bs_prettify",
        action="store_true",
        help="old html processing behaviour (before v0.5.5). "
        "Sometimes it will yield better results than the new default.",
        advanced=True,
        caption="BeautifulSoup prettify",
    )

    cmdcompose = subparsers.add_parser("compose")
    cmdcompose.add_argument(
        "--merge",
        action="store_true",
        help="merge several source files before output.",
        advanced=True,
        hide=True,
    )
    cmdcompose.add_argument(
        "--nots",
        action="store_true",
        help="don't append timestamp to filenames",
        caption="Не добавлять временную отметку в имя файла",
        advanced=True,
    )
    cmdcompose.add_argument(
        "--labels_file",
        help="i18n config",
        caption="Конфиг для интернационализации",
        advanced=True,
        argtype="filename",
    )
    cmdcompose.add_argument(
        "--imgur_client_id",
        help="imgur client id",
        caption="Client ID для API Imgur",
        advanced=True,
    )
    cmdcompose_filetype = cmdcompose.add_subparsers(dest="filetype")
    cmdcompose_docx = cmdcompose_filetype.add_parser("docx")
    cmdcompose_docx.add_argument(
        "--docx_template",
        help="a DocX template file.",
        advanced=True,
        caption="Файл-образец",
        argtype="filename",
    )
    cmdcompose_docx.add_argument(
        "filename",
        nargs="*",
        help="file(s) to compose from.",
        caption="Имя 4s-файла",
        filetypes=[("chgksuite markup files", "*.4s")],
    )
    cmdcompose_docx.add_argument(
        "--nospoilers",
        "-n",
        action="store_true",
        help="do not whiten (spoiler) answers.",
        caption="Не забелять ответы",
    )
    cmdcompose_docx.add_argument(
        "--noanswers",
        action="store_true",
        help="do not print answers " "(not even spoilered).",
        caption="Без ответов",
    )
    cmdcompose_docx.add_argument(
        "--noparagraph",
        action="store_true",
        help="disable paragraph break " "after 'Question N.'",
        advanced=True,
        caption='Без переноса строки после "Вопрос N."',
    )
    cmdcompose_docx.add_argument(
        "--randomize",
        action="store_true",
        help="randomize order of questions.",
        advanced=True,
        caption="Перемешать вопросы",
    )
    cmdcompose_docx.add_argument(
        "--no_line_break",
        action="store_true",
        help="no line break between question and answer.",
        caption="Один перенос строки перед ответом вместо двух",
    )
    cmdcompose_docx.add_argument(
        "--one_line_break",
        action="store_true",
        help="one line break after question instead of two.",
        caption="Один перенос строки после вопроса вместо двух",
    )

    cmdcompose_tex = cmdcompose_filetype.add_parser("tex")
    cmdcompose_tex.add_argument(
        "--tex_header",
        help="a LaTeX header file.",
        caption="Файл с заголовками",
        advanced=True,
        argtype="filename",
    )
    cmdcompose_tex.add_argument(
        "filename",
        nargs="*",
        help="file(s) to compose from.",
        caption="Имя 4s-файла",
        filetypes=[("chgksuite markup files", "*.4s")],
    )
    cmdcompose_tex.add_argument(
        "--rawtex",
        action="store_true",
        advanced=True,
        caption="Не удалять исходный tex",
    )

    cmdcompose_lj = cmdcompose_filetype.add_parser("lj")
    cmdcompose_lj.add_argument(
        "filename",
        nargs="*",
        help="file(s) to compose from.",
        caption="Имя 4s-файла",
        filetypes=[("chgksuite markup files", "*.4s")],
    )
    cmdcompose_lj.add_argument(
        "--nospoilers",
        "-n",
        action="store_true",
        help="disable spoilers.",
        caption="Отключить спойлер-теги",
    )
    cmdcompose_lj.add_argument(
        "--splittours",
        action="store_true",
        help="make a separate post for each tour.",
        caption="Разбить на туры",
    )
    cmdcompose_lj.add_argument(
        "--genimp",
        action="store_true",
        help="make a 'general impressions' post.",
        caption="Пост с общими впечатлениями",
    )
    cmdcompose_lj.add_argument(
        "--navigation",
        action="store_true",
        help="add navigation to posts.",
        caption="Добавить навигацию к постам",
    )
    cmdcompose_lj.add_argument(
        "--login", "-l", help="livejournal login", caption="ЖЖ-логин"
    )
    cmdcompose_lj.add_argument(
        "--password", "-p", help="livejournal password", caption="Пароль от ЖЖ"
    )
    cmdcompose_lj.add_argument(
        "--community",
        "-c",
        help="livejournal community to post to.",
        caption="ЖЖ-сообщество",
    )
    cmdcompose_lj.add_argument(
        "--security",
        help="set to 'friends' to make post friends-only, else specify allowmask.",
        caption="Указание группы друзей (или 'friends' для всех друзей)",
    )
    cmdcompose_base = cmdcompose_filetype.add_parser("base")
    cmdcompose_base.add_argument(
        "filename",
        nargs="*",
        help="file(s) to compose from.",
        caption="Имя 4s-файла",
        filetypes=[("chgksuite markup files", "*.4s")],
    )
    cmdcompose_base.add_argument(
        "--remove_accents",
        action="store_true",
        caption="Убрать знаки ударения",
        help="remove combining acute accents to prevent db.chgk.info search breaking",
    )
    cmdcompose_base.add_argument(
        "--clipboard",
        caption="Скопировать результат в буфер",
        help="copy result to clipboard",
        action="store_true",
    )
    cmdcompose_redditmd = cmdcompose_filetype.add_parser("redditmd")
    cmdcompose_redditmd.add_argument(
        "filename",
        nargs="*",
        help="file(s) to compose from.",
        caption="Имя 4s-файла",
        filetypes=[("chgksuite markup files", "*.4s")],
    )
    cmdcompose_pptx = cmdcompose_filetype.add_parser("pptx")
    cmdcompose_pptx.add_argument(
        "filename",
        nargs="*",
        help="file(s) to compose from.",
        caption="Имя 4s-файла",
        filetypes=[("chgksuite markup files", "*.4s")],
    )
    cmdcompose_pptx.add_argument(
        "--pptx_config",
        help="a pptx config file.",
        advanced=True,
        caption="Файл конфигурации",
        argtype="filename",
    )

    cmdtrello = subparsers.add_parser("trello")
    cmdtrello_subcommands = cmdtrello.add_subparsers(dest="trellosubcommand")
    cmdtrello_download = cmdtrello_subcommands.add_parser(
        "download", caption="Скачать из Трелло"
    )
    cmdtrello_download.add_argument(
        "folder",
        help="path to the folder" "to synchronize with a trello board.",
        caption="Папка",
    )
    cmdtrello_download.add_argument(
        "--si",
        action="store_true",
        help="This flag includes card captions "
        "in .4s files. "
        "Useful for editing SI "
        "files (rather than CHGK)",
        caption="Формат Своей игры",
    )
    cmdtrello_download.add_argument(
        "--onlyanswers",
        action="store_true",
        help="This flag forces SI download to only include answers.",
        caption="Только ответы",
    )
    cmdtrello_download.add_argument(
        "--noanswers",
        action="store_true",
        help="This flag forces SI download to not include answers.",
        caption="Без ответов",
    )
    cmdtrello_download.add_argument(
        "--singlefile",
        action="store_true",
        help="This flag forces SI download all themes to single file.",
        caption="Склеить всё в один файл",
    )
    cmdtrello_download.add_argument(
        "--qb", action="store_true", help="Quizbowl format", caption="Формат квизбола"
    )
    cmdtrello_download.add_argument(
        "--labels",
        action="store_true",
        help="Use this if you also want " "to have lists based on labels.",
        caption="Создать файлы из лейблов Трелло",
    )

    cmdtrello_upload = cmdtrello_subcommands.add_parser(
        "upload", caption="Загрузить в Трелло"
    )
    cmdtrello_upload.add_argument(
        "board_id", help="trello board id.", caption="ID доски"
    )
    cmdtrello_upload.add_argument(
        "filename",
        nargs="*",
        help="file(s) to upload to trello.",
        caption="Имя 4s-файла",
    )
    cmdtrello_upload.add_argument(
        "--author",
        action="store_true",
        help="Display authors in cards' captions",
        caption="Дописать авторов в заголовок карточки",
    )

    cmdtrello_subcommands.add_parser("token")

    args = parser.parse_args()

    if not args.labels_file:
        args.labels_file = os.path.join(resourcedir, "labels_ru.toml")
    if not args.regexes:
        args.regexes = os.path.join(resourcedir, "regexes.json")
    if not args.docx_template:
        args.docx_template = os.path.join(resourcedir, "template.docx")
    if not args.pptx_config:
        args.pptx_config = os.path.join(resourcedir, "pptx_config.toml")
    if not args.tex_header:
        args.tex_header = os.path.join(resourcedir, "cheader.tex")
    if args.config:
        with open(args.config, "r") as f:
            config = json.load(f)
        for key in config:
            if not isinstance(config[key], basestring):
                val = config[key]
            elif os.path.isfile(config[key]):
                val = os.path.abspath(config[key])
            elif os.path.isfile(
                os.path.join(os.path.dirname(os.path.abspath(__file__)), config[key])
            ):
                val = os.path.join(
                    os.path.dirname(os.path.abspath(__file__)), config[key]
                )
            else:
                val = config[key]
            setattr(args, key, val)

    args.passthrough = False
    if not args.action:
        try:
            ret = gui_choose_action(args)
            action = ret["action"]
            defaultauthor = ret["defaultauthor"]
            merge = ret["merge"]
            passthrough = ret["passthrough"]
        except ValueError:
            sys.exit(1)
        if passthrough:
            args.passthrough = True
        if action == "parse":
            args.action = "parse"
            args.defaultauthor = defaultauthor
        if action == "parsedir":
            args.action = "parse"
            args.defaultauthor = defaultauthor
            args.parsedir = True
        if action == "compose":
            args.action = "compose"
            args.merge = merge
        if action == "trellodown":
            args.action = "trello"
            args.trellosubcommand = "download"
        if action == "trelloup":
            args.action = "trello"
            args.trellosubcommand = "upload"
        if action == "trellotoken":
            args.action = "trello"
            args.trellosubcommand = "token"
    if args.action == "parse":
        gui_parse(args, sourcedir=sourcedir)
    if args.action == "compose":
        gui_compose(args, sourcedir=sourcedir)
    if args.action == "trello":
        gui_trello(args, sourcedir=sourcedir)



#!usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re
try:
    import urllib
    unquote = urllib.unquote
except AttributeError:
    import urllib.parse
    unquote = urllib.parse.unquote_to_bytes
try:
    basestring
except NameError:
    basestring = str

OPENING_QUOTES = set(['«', '„', '“'])
CLOSING_QUOTES = set(['»', '“', '”'])
QUOTES = OPENING_QUOTES | CLOSING_QUOTES | set(['"', "'"])
WHITESPACE = set([' ', ' ', '\n'])
PUNCTUATION = set([',', '.', ':', ';', '?', '!'])
OPENING_BRACKETS = ['[', '(', '{']
CLOSING_BRACKETS = [']', ')', '}']
BRACKETS = set(OPENING_BRACKETS) | set(CLOSING_BRACKETS)
LOWERCASE_RUSSIAN = set(list('абвгдеёжзийклмнопрстуфхцчшщъыьэюя'))
UPPERCASE_RUSSIAN = set(list('АБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ'))
POTENTIAL_ACCENTS = set(list('АОУЫЭЯЕЮИ'))
BAD_BEGINNINGS = set(['Мак', 'мак', "О'", 'о’', 'О’', "о'"])
NO_BREAK_SEQUENCES = [
    "а",
    "без",
    "в",
    "где",
    "для",
    "же",
    "за",
    "и",
    "или",
    "из",
    "из-за",
    "к",
    "как",
    "на",
    "над",
    "не",
    "ни",
    "но",
    "о",
    "от",
    "по",
    "под",
    "при",
    "с",
    "то",
    "у",
    "что",
    "перед"
]
NO_BREAK_SEQUENCES_LEFT = [
    "бы",
    "ли",
    "же"
]

re_bad_wsp_start = re.compile(r'^[{}]+'.format(''.join(WHITESPACE)))
re_bad_wsp_end = re.compile(r'[{}]+$'.format(''.join(WHITESPACE)))
re_url = re.compile(r"""((?:[a-z][\w-]+:(?:/{1,3}|[a-z0-9%])|www\d{0,3}[.]"""
                    """|[a-z0-9.\-]+[.‌​][a-z]{2,4}/)(?:[^\s()<>]+|(([^\s()<>]+|(([^\s()<>]+)))*))+"""
                    """(?:(([^\s()<>]+|(‌​([^\s()<>]+)))*)|[^\s`!()[]{};:'".,<>?«»“”‘’]))""",
                    re.DOTALL)
re_percent = re.compile(r"(%[0-9a-fA-F]{2})+")


def remove_excessive_whitespace(s):
    s = re_bad_wsp_start.sub('', s)
    s = re_bad_wsp_end.sub('', s)
    s = re.sub(r'\s+\n\s+', '\n', s)
    return s


def get_quotes_right(s_in):
    s = s_in
    s = re.sub(r'[{}]'.format(''.join(OPENING_QUOTES)), '«', s)
    s = re.sub(r'[{}]'.format(''.join(CLOSING_QUOTES)), '»', s)
    s = re.sub(r'^"', '«', s)
    s = re.sub(r'"$', '»', s)
    s = re.sub(r'(\w)(")([^\w]|$)', r'\1»\3', s, flags=re.U)
    s = re.sub(r'(^|[^\w])(")(\w)', r'\1«\3', s, flags=re.U)
    s = re.sub(r'([^\w\s])(")([^\w\s])', r'\1»\3', s, flags=re.U)
    s = re.sub(r'" ', '» ', s)
    s = re.sub(r' "', ' «', s)
    cntr = 0

    s = list(s)
    for i, c in enumerate(s):
        if c == '"':
            if cntr:
                s[i] = '»'
                cntr -= 1
            else:
                s[i] = '«'
                cntr += 1

    cntr = 0
    for i, c in enumerate(s):
        if c == '«':
            cntr += 1
            if cntr % 2 == 0:
                s[i] = '„'
        if c == '»':
            if cntr % 2 == 0:
                s[i] = '“'
            cntr -= 1
    s = ''.join(s)

    s = re.sub(r"(\w)'", r'\1’', s, flags=re.U)
    s = re.sub(r"'(\w)", r'‘\1', s, flags=re.U)

    return s


def get_dashes_right(s):
    s = re.sub(r'(?<=\s)-+(?=\s)', '—', s)
    # s = re.sub(r'(?<=\d)-(?<=\d)','–',s)
    # s = re.sub(r'-(?=\d)','−',s)
    return s


def replace_no_break_spaces(s):
    for sp in (NO_BREAK_SEQUENCES + [x.title() for x in NO_BREAK_SEQUENCES]):
        r_from = "([ \u00a0]){sp} ".format(sp=sp)
        r_to = "\g<1>{sp}\u00a0".format(sp=sp)
        s = re.sub(r_from, r_to, s)
    for sp in (NO_BREAK_SEQUENCES_LEFT + [x.title() for x in NO_BREAK_SEQUENCES_LEFT]):
        r_from = " {sp}([ \u00a0])".format(sp=sp)
        r_to = "\u00a0{sp}\g<1>".format(sp=sp)
        s = re.sub(r_from, r_to, s)
    return s


def search_accent(s):
    return re.search("([а-яё])([АОУЫЭЯЕЮИ])([^А-ЯЁ])", s) or re.search(
        "[^А-ЯЁ]([А-ЯЁ])([АОУЫЭЯЕЮИ])([^А-ЯЁ])", s
    )


def detect_accent(s):
    # srch = search_accent(s)
    # while srch:
    #     to_replace = srch.group(1) + srch.group(2) + srch.group(3)
    #     replacement = "{}{}\u0301{}".format(
    #         srch.group(1), srch.group(2).lower(), srch.group(3)
    #     )
    #     s = s.replace(to_replace, replacement)
    #     srch = search_accent(s)
    # return s
    for word in re.split(r'[^{}{}]+'.format(
            ''.join(LOWERCASE_RUSSIAN), ''.join(UPPERCASE_RUSSIAN)), s):
        if word.upper() != word and len(word) > 1:
            try:
                i = 1
                word_new = word
                while i < len(word_new):
                    if (word_new[i] in POTENTIAL_ACCENTS and
                            word_new[:i] not in BAD_BEGINNINGS):
                        word_new = word_new[:i] + \
                            word_new[i].lower() + '\u0301' + word_new[i + 1:]
                    i += 1
                if word != word_new:
                    s = (s[:s.index(word)] + word_new +
                         s[s.index(word) + len(word):])
            except:
                print(repr(word))
    return s


def percent_decode(s):
    grs = sorted([match.group(0)
                  for match in re_percent.finditer(s)], key=len, reverse=True)
    for gr in grs:
        try:
            s = s.replace(gr, unquote(gr.encode('utf8')).decode('utf8'))
        except:
            pass
    return s


def recursive_typography(s):
    if isinstance(s, basestring):
        s = typography(s)
        return s
    elif isinstance(s, list):
        new_s = []
        for element in s:
            new_s.append(recursive_typography(element))
        return new_s


def typography(s, wsp=True, quotes=True,
               dashes=True, accents=True, percent=True):
    if wsp:
        s = remove_excessive_whitespace(s)
    if quotes:
        s = get_quotes_right(s)
    if dashes:
        s = get_dashes_right(s)
    if accents:
        s = detect_accent(s)
    if percent:
        s = percent_decode(s)
    return s


def matching_bracket(s):
    assert s in OPENING_BRACKETS or s in CLOSING_BRACKETS
    if s in OPENING_BRACKETS:
        return CLOSING_BRACKETS[OPENING_BRACKETS.index(s)]
    return OPENING_BRACKETS[CLOSING_BRACKETS.index(s)]


def find_matching_closing_bracket(s, index):
    s = list(s)
    i = index
    assert s[i] in OPENING_BRACKETS
    ob = s[i]
    cb = matching_bracket(ob)
    counter = 0
    while i < len(s):
        if s[i] == ob:
            counter += 1
        if s[i] == cb:
            counter -= 1
            if counter == 0:
                return i
        i += 1
    return None


def find_matching_opening_bracket(s, index):
    s = list(s)
    i = index
    assert s[i] in CLOSING_BRACKETS
    cb = s[i]
    ob = matching_bracket(cb)
    counter = 0
    if i < 0:
        i = len(s) - abs(i)
    while i < len(s) and i >= 0:
        if s[i] == cb:
            counter += 1
        if s[i] == ob:
            counter -= 1
            if counter == 0:
                return i
        i -= 1
    return None
